import { SelectField } from "./select-field";
import { SelectEntry } from "./select-entry";
import { FormulaireNode } from "./formulaire-node";

/**
 * A select field that populates itself with references to
 * available objects (for ex. Nub or ParamDefinition)
 */
export abstract class SelectFieldReference extends SelectField {

    /** source identifier for populate() method */
    protected _source: string;

    constructor(parent: FormulaireNode, source: string) {
        super(parent);
        this._source = source;
    }

    protected abstract initSelectedValue();

    /**
     * Populates entries with available references
     */
    protected abstract populate();

    /**
     * Once config is parsed, init original value from model
     * (needs config, for this._entriesBaseId to be set)
     */
    protected afterParseConfig() {
        this.populate();
        this.initSelectedValue();
    }

    /**
     * Reloads available entries, trying to keep the current selected
     * value; does not notify observers if value did not change
     */
    public updateEntries() {
        // store previous selected entry
        const pse = this._selectedEntry;
        // empty
        this.clearEntries();
        // populate
        this.populate();
        // if no entry is available anymore, unset value
        if (this.entries.length === 0) {
            super.setValue(undefined);
        } else {
            // keep previously selected entry if possible
            if (pse && pse.id) {
                this.setValueFromId(pse.id);
            } else {
                this.setDefaultValue();
            }
        }
    }

    /**
     * Updates selectedValue; notifies observers only if
     * value.id has changed
     */
    public setValue(v: SelectEntry) {
        const previousSelectedEntry = this._selectedEntry;
        this._selectedEntry = v;
        if (
            ! previousSelectedEntry
            || (previousSelectedEntry.id !== v.id)
        ) {
            this.notifySelectValueChanged();
        }
    }

    public notifySelectValueChanged() {
        this.notifyObservers({
            "action": "select",
            "value": this._selectedEntry
        }, this);
    }

    /**
     * Sets value from given ID; if it was not found, sets the
     * first available entry as selectedValue
     */
    public setValueFromId(id: string) {
        let found = false;
        for (const e of this._entries) {
            if (e.id === id) {
                found = true;
                this.setValue(e);
            }
        }
        if (! found) {
            this.setDefaultValue();
        }
    }

    protected setDefaultValue() {
        // default to first available entry if any
        if (this._entries.length > 0) {
            this.setValue(this._entries[0]);
        } else {
            // notify observers that no value is selected anymore
            this.notifySelectValueChanged();
        }
    }
}
